#include "atol_interface.h"

#pragma comment(linker,"/NODEFAULTLIB")
#pragma comment(linker,"/ENTRY:DllMain")
#pragma comment(linker,"/FILEALIGN:512")
#pragma comment(linker,"/MERGE:.data=.text")
#pragma comment(linker,"/MERGE:.rdata=.text")
#pragma comment(linker,"/SECTION:.text,ERWX")
#pragma comment(linker,"/IGNORE:4078")

// atol_file (C) 2005, Alexander G. Karpov [mrhx]

#define f(name) (*IAtol->name)

BOOL WINAPI DllMain(HINSTANCE, DWORD, LPVOID)
{
	return TRUE;
}

const char * ExportList [] =	// EXPORT LIST
{
	"file.open",			"f_open",
	"file.close",			"f_close",
	"file.write",			"f_write",
	"file.writeln",			"f_writeln",
	"file.writenl",			"f_writeln",
	"file.nl",				"f_nl",
	"file.readchar",		"f_readch",
	"file.readch",			"f_readch",
	"file.readln",			"f_readln",
	"file.readline",		"f_readln",
	NULL
};

const char ** WINAPI atol_lib_install(ATOL_INTERFACE IAtol)
{
	f(error)(lstrcmp( f(version)(), "0.9") < 0, "Need ATOL version 0.9 or above");
	return ExportList;
}

ATOL_ATOM WINAPI f_open(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // open a file.
// file.open( filename, mode ) returns handle.
{
	f(error)(2 != f(list_length)(args), "file.open requires two arguments: filename and mode ('read', 'write' or 'all')");

	ATOL_STRING mode = f(list_item)(f(list_next)(args));

	static const struct { char * name; DWORD access, share, creation; } modes [4] =
	{
		"r",  GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING,
		"w", GENERIC_WRITE, 0, OPEN_ALWAYS,
		"rw",	 GENERIC_WRITE|GENERIC_READ, 0, OPEN_ALWAYS,
		"wr",	 GENERIC_WRITE|GENERIC_READ, 0, OPEN_ALWAYS
	};
	HANDLE hFile = INVALID_HANDLE_VALUE;

	for (register int i = 0; i < 4; ++i)
		if (!lstrcmp(mode, modes[i].name))
		{
			hFile = CreateFile(f(list_item)(args), modes[i].access, modes[i].share,
						NULL, modes[i].creation, FILE_ATTRIBUTE_NORMAL, NULL);
			break;
		}

	f(error)(hFile == INVALID_HANDLE_VALUE, "file.open: cannot use file or wrong mode");

	return f(atomi)((int) hFile);
}

ATOL_ATOM WINAPI f_close(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // open a file.
{
	f(error)(1 != f(list_length)(args), "file.close requires one argument");
	CloseHandle((HANDLE) f(atoi)( f(list_item)(args)));
	return f(null)();
}

ATOL_ATOM WINAPI f_readch(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // read char from file.
// file.readchar(f)
{
	f(error)(f(list_length)(args) != 1, "file.readchar requires one argument");
	char ch [2] = { 0, 0 };
	DWORD d;
	f(error)(!ReadFile((HANDLE) f(atoi)( f(list_item)(args)), ch, 1, &d, NULL), "file.readchar: fail");
	if (!d) return f(atom)("EOF");
	return f(atom)(ch);
}

ATOL_ATOM WINAPI f_readln(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // read line from file.
// file.readln(+f)
{
	f(error)(f(list_length)(args) != 1, "file.readln requires one argument");
	char str [1024];
	DWORD d;
	register int i = 0;
	register HANDLE hFile = (HANDLE) f(atoi)( f(list_item)(args));
	do
	{
		f(error)(!ReadFile(hFile, str+i, 1, &d, NULL), "file.readln: fail");
		if (!d || str[i] == '\n')
		{
			if (i > 0 && str [i-1] == '\r') --i;
			break;
		}
		++i;
		if (i >= 1023) break;
	}
	while (1);
	str [i] = 0;
	if (!i && !d) return f(atom)("EOF");
	return f(atom)(str);
}

ATOL_ATOM WINAPI f_nl(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // write into file.
// file.nl(f)
{
	f(error)(f(list_length)(args) < 1, "file.nl requires 1 argument");
	DWORD d;
	f(error)(!WriteFile((HANDLE) f(atoi)( f(list_item)(args)), "\r\n", 2, &d, NULL), "file.nl: fail");
	return f(list_item_atom)(args);
}

ATOL_ATOM WINAPI f_write(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // write into file.
// file.write(f atom(s)...)
{
	f(error)(f(list_length)(args) <= 1, "file.write requires > 1 arguments");
	DWORD d;
	register HANDLE hFile = (HANDLE) f(atoi)( f(list_item)(args));
	for (register ATOL_LIST p = f(list_next)(args); p; p = f(list_next)(p))
	{
		f(error)(!WriteFile(hFile, f(list_item)(p), lstrlen(f(list_item)(p)), &d, NULL),
				 "file.write: fail");
	}
	return f(list_item_atom)(args);
}

ATOL_ATOM WINAPI f_writeln(ATOL_INTERFACE IAtol, ATOL_LIST args, ATOL_LIST params) // write into file.
// file.writeln(f ...)
{
	f_write(IAtol, args, params);
	return f_nl(IAtol, args, params);
}

