//------------------------------------------------------------------
// INCLUDED FILE.
//------------------------------------------------------------------
// This contents functions to support window.
//------------------------------------------------------------------

LRESULT CALLBACK win_proc(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	register ATOL_INTERFACE IAtol = (ATOL_INTERFACE) GetWindowLong(This, WNDX_IATOL);

	if (IAtol) f(set_context)("atol_gui:wnd", This);

	switch (Msg)
	{
	case WM_COMMAND:
		switch (HIWORD(wParam))
		{
		case BN_CLICKED:
//		case STN_CLICKED:
			CallHook(IAtol, (HWND) lParam, "click");
			break;
		}
		break;

//	case WM_CREATE:
//		SetFocus(GetWindow(This, GW_CHILD));
//		break;

	case WM_GETFONT:
		return GetWindowLong(This, WNDX_FONT);

	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(This, Msg, wParam, lParam);
	}
	return FALSE;
}

void CheckWindowSize(HWND hParent)
{
	RECT rc, RC;
	int cx = 0, cy = 0;
	for (register HWND h = GetWindow(hParent, GW_CHILD); h; h = GetWindow(h, GW_HWNDNEXT))
	{
		GetWindowRect(h, &rc);
		MapWindowPoints(NULL, hParent, (LPPOINT) &rc, 2);
		if (rc.right > cx) cx = rc.right;
		if (rc.bottom > cy) cy = rc.bottom;
	}
	GetWindowRect(hParent, &RC);
	GetClientRect(hParent, &rc);

	int W = cx + RC.right - RC.left - rc.right + DX;
	int H = cy + RC.bottom - RC.top - rc.bottom + DY;
	int f = GetWindowLong(hParent, WNDX_FLAGS);

	SetWindowPos(hParent, NULL, 0, 0,
		(f & WNDX_FLAG_PRESETCX)?(RC.right - RC.left):W,
		(f & WNDX_FLAG_PRESETCY)?(RC.bottom - RC.top):H,
		SWP_NOZORDER | SWP_NOMOVE);
}

/******************* EXTERNAL *******************************************************/

ATOL_API(win)		// new window.
// gui.window( +Name +Title )
{
	check_args(IAtol, args, 2, "window");

	WNDCLASS wc;

	wc.cbClsExtra = 0;
	wc.cbWndExtra = WNDX_SIZE;
	wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hInstance = GetModuleHandle(NULL);
	wc.lpfnWndProc = win_proc;
	wc.lpszClassName = WNDCLSNAME;
	wc.lpszMenuName = NULL;
	wc.style = CS_DBLCLKS | CS_PARENTDC;

	RegisterClass(&wc); // register wnd class.

	int width = GetWidth(IAtol), height = GetHeight(IAtol);

	HWND hwnd = CreateWindowEx(WS_EX_CONTROLPARENT, WNDCLSNAME,
					f(list_get)(args, 1),
					WS_OVERLAPPEDWINDOW ^ WS_MAXIMIZEBOX ^ WS_SIZEBOX,
					CW_USEDEFAULT, CW_USEDEFAULT,
					width?width:GetSystemMetrics(SM_CXSCREEN),
					height?height:GetSystemMetrics(SM_CYSCREEN),
					NULL, NULL, wc.hInstance, NULL);

	f(error)(!hwnd, "gui.window: Could not create a window");

	f(set_context)("atol_gui:wnd", hwnd); // set context.

	// FILL WND-EXTRA BYTES:

	SetWindowLong(hwnd, WNDX_IATOL, (LONG) IAtol); // store IAtol for wnd_proc.
	SetWindowLong(hwnd, WNDX_FONT, (LONG)			// store window font.
		CreateFont(-13, 0, 0, 0, 400, 0, 0, 0, 204, 3, 2, 1, 18, "Times New Roman"));
	SetWndName(IAtol, hwnd, f(list_item)(args)); // Set NAME of this window.

	SetWindowLong(hwnd, WNDX_FLAGS,
		(width?WNDX_FLAG_PRESETCX:0) | (height?WNDX_FLAG_PRESETCY:0)); // FLAGS.

	CallHook(IAtol, hwnd, "create");   // Call OnCreate HOOK.

	return f(atomi)((int) hwnd); // return window handle.
}

ATOL_API(end)		// ends window.
// gui.end
{
	register HWND hParent = GetCurrentWnd(IAtol);

	CheckWindowSize(hParent);

	// CENTER WINDOW:
	RECT rc, RC;
	HWND hDesktop = GetDesktopWindow();
	GetWindowRect(hParent, &rc);
	GetClientRect(hDesktop, &RC);

	SetWindowPos(hParent, NULL,
		(RC.right - rc.right + rc.left) / 2,
		(RC.bottom - rc.bottom + rc.top) / 2,
		0, 0, SWP_NOZORDER | SWP_NOSIZE);

	ShowWindow(hParent, SW_SHOWNORMAL);		// SHOW WINDOW.
	UpdateWindow(hParent);

	return f(null)();
}

